#Alte Programme werden gelscht#
rm(euler, eulersml)

#Die in C Programmierten Simulated Maximum Likelihood Programme werden per .dll Datei eingebunden#
dyn.load("euler.dll")

#Um die Funktionen in R nutzen zu knnen mssen sie noch definiert werden#
euler<-function(startwert, deltat, steps, mu, muparameter, sigma, sigmaparameter, seed, nobs)
  .C("euler",
     as.double(startwert),
     as.double(deltat),
     as.integer(steps),
     as.integer(mu),
     as.double(muparameter),
     as.integer(sigma),
     as.double(sigmaparameter),
     as.integer(seed),
     as.integer(nobs-1),
     x=double(nobs))$x

eulersml<-function(x, deltat, steps, n, mu, muparameter, sigma, sigmaparameter, seed)
  .C("euler_sml",
     as.double(x),
     as.integer(length(x)),
     as.double(deltat),
     as.integer(steps),
     as.integer(n),
     as.integer(mu),
     as.double(muparameter),
     as.integer(sigma),
     as.double(sigmaparameter),
     as.integer(seed),
     x=double(1))$x

# Das SML Programm wir so umgewandelt, dass nach allen drei Parametern (para) optimiert werden kann#
eulersml1<-function(para,x, deltat, steps, n, mu, sigma, seed)
    {
    theta2 <- abs(para[1])#kappa
        theta1 <- abs(para[2])#theta
        theta3 <- abs(para[3])#sigma
        mu_par<-c(theta2,theta1)
    ausg<-eulersml(x, deltat, steps, n, mu,mu_par,sigma,theta3, seed)
    return(-ausg)
    }

#Maximierung der SML Likelihood#
mineulersml <- function(parameter,x, deltat, steps, n, mu, sigma, seed)
  {
    optim(parameter,eulersml1,gr=NULL,"Nelder-Mead",lower=-Inf, upper=Inf, list(),hessian=FALSE,x, deltat, steps, n, mu, sigma, seed)
  }


#Simulation mit der Euler Methode#
simulateeuler <-function(kappa,theta,sigma,delta,L,x0)
{

ts<-matrix(ncol=(L/delta + 1),nrow=2,NA)
ts[2,1]<-x0
l<-0:(L/delta)
l<-l*delta
ts[1,]<-l
c<-rnorm(length(ts[2,]))
for (i in 2:length(ts[2,])){
    delta1<-ts[1,i]-ts[1,i-1]
    ts[2,i]<- ts[2,i-1] + kappa*(theta-ts[2,i-1])*delta1 + sigma*sqrt(delta1*abs(ts[2,i-1]))*c[i]
}
return(ts)
}

#Likelihood der Euler Methode#
lleuler<-function(para,t,x)
  {
    ll <- 0.0
    theta2 <- abs(para[1])#kappa
    theta1 <- abs(para[2])#theta
    theta3 <- abs(para[3])#sigma
    nobs <- length(t)
    for (i in 2:nobs)
         {
        delta <- t[i]-t[i-1]
        ll<-ll+log(dnorm(x[i],mean=x[i-1]+theta2*(theta1-x[i-1])*delta,sd=theta3*sqrt(delta*abs(x[i-1]))))



         }
    -ll
  }

#Maximierung der Euler Likelihood#
mineulerll <- function(parameter,t,x)
  {
    optim(parameter,lleuler,gr=NULL,"Nelder-Mead",lower=-Inf, upper=Inf, list(),hessian=FALSE,t,x)
  }


#Simulation mit der begangsdichte des Square-Root Prozesses#
simulatesrp <- function(startwert,kappa,theta,sigma,t)
  {
    nobs <- length(t)
    out <- vector(mode="numeric",length = nobs)
    out[1] <- startwert
    for(i in 2:nobs)
      {
        delta <- t[i]-t[i-1]
        c <- (2.0*kappa)/((sigma*sigma)*(1.0-exp(-kappa*delta)))
        out[i] <- rchisq(1,df=(4.0*kappa*theta)/(sigma*sigma),ncp=2.0*c*out[i-1]*exp(-kappa*delta))/(2.0*c)
      }
    out
  }

#Likelihood des Square-Root Prozesses# 
llsrp <- function(para,t,x)
  {
    ll <- 0.0
    theta2 <- abs(para[1])#kappa
    theta1 <- abs(para[2])#theta
    theta3 <- abs(para[3])#sigma
    nobs <- length(t)
    for (i in 2:nobs)
      {
        delta <- t[i]-t[i-1]
        c <- 2*theta2/(theta3^2*(1-exp(-theta2*delta)))
        u <- c*x[i-1]*exp(-theta2*delta)
        v <- c*x[i]
        q <- 2*theta2*theta1/(theta3^2)-1
        ll <- ll+log(c) + (-u-v) + log((v/u)^(0.5*q))+ (2*sqrt(u*v)) + log(besselI(2*sqrt(u*v),q,expon.scaled=TRUE))
        #ll <- ll+log(c*exp(-u-v)*(v/u)^(0.5*q)*exp(2*sqrt(u*v))*besselI(2*sqrt(u*v),q,expon.scaled=TRUE))
      }
    -ll

  }

#Maximierung der Likelihood des Square-Root Prozesses#
minsrp <- function(parameter,t,x)
  {
    optim(parameter,llsrp,gr=NULL,"Nelder-Mead",lower=-Inf, upper=Inf, list(),hessian=FALSE,t,x)
  }



